#$Id: Daemon.pm,v 1.2 2004/08/19 01:18:22 fil Exp $
########################################################
package POE::Component::Daemon;

use 5.00405;
use strict;

use vars qw($VERSION @ISA);

use POSIX qw(EAGAIN ECHILD SIGINT SIGKILL SIGTERM);
use POE;
use Carp;
use Data::Dumper;

use POE::Component::Daemon::Scoreboard;

$VERSION = '0.02';

sub DEBUG () { 0 }
sub DEBUG_SC () { 0 }

########################################################
sub new
{
    my($package)=shift;
    my $param;
    if(1==@_) {
        $param=shift;
    }
    else {
        $param={@_};
    }

    my $self=bless $param, $package;
    $self->{package}=$package;

    $package =~ s/\W+/-/g;
    $self->{alias}||=$package;
    return $self;
}

########################################################
sub spawn
{
    my($self)=shift;
    my $param={@_};

    unless(ref $self) {
        $self=$self->new($param);
    }
    else {
        while(my($k, $v)=each %$param) {
            $self->{$k}=$v if defined $v;
        }
    }

    $self->open_logfile;
    $self->drop_privs;
    $self->detach;

    return $self->create_session;
}

########################################################
sub drop_privs
{
    my($self)=@_;
    return if $self->{"drop_privs done"}++;
    return unless $self->{UID} or $self->{GID};
    warn ref($self), "->drop_prives TODO";
}


########################################################
sub open_logfile
{
    my($self)=@_;
    my $logfile=$self->{logfile};
    return unless $logfile;
    return if $self->{"logfile done"}++;
    open STDOUT, ">>$logfile" or die "Unable to write to $logfile: $!\n";   
    open STDERR, ">&STDOUT"   or die "Unable to reopen STDERR: $!\n";
    STDERR->autoflush(1);
    STDOUT->autoflush(1);
    return 1;
}

########################################################
sub detach
{
    my($self)=@_;
    return if $self->{"detach done"}++;
    return unless $self->{detach};

    my $gen="parent";

        DEBUG and warn "Detaching from $gen";
    my $pid=fork;
    die "Unable to fork: $!\n" unless defined $pid;
    if(0==$pid) {                       # child
        $gen="child";
        DEBUG and warn "Detaching from $gen";
        $pid=fork;
        die "Unable to fork: $!\n" unless defined $pid;
        if(0==$pid) {                   # grand-child
            $gen="grand-child";
            DEBUG and warn "We are the $gen";
            POSIX::setsid() or 
                warn "Unable to setsid(): $! (continuing anyway)";
            return 1;
         }
    }
    DEBUG and warn "I am the $gen, and now I exit.";
    ## we are parent or child.  Exit
    $POE::Kernel::kr_run_warning |= POE::Kernel::KR_RUN_CALLED;
    exit 0;
}

########################################################
sub create_session
{
    my($self)=@_;

    POE::Session->create(
            object_states => [
                $self=>[qw(
                        _start _stop status update_status
                        check_scoreboard
                        fork retry waste_time
                        babysit rogues shutdown
                        sig_CHLD sig_INT
                        )]
            ]);
}

########################################################
sub is_prefork
{
    my($self)=@_;
    return 0 != ($self->{start_children}||0);
}

########################################################
sub is_fork
{
    my($self)=@_;
    return 0 != ($self->{max_children}||0);
}


########################################################
sub default_min_max
{
    my($self)=@_;
    if($self->{max_children}) {
        if($self->{max_spare}) {
            $self->{min_spare} ||= int($self->{max_spare} /2);
        }
        else {
            $self->{min_spare} ||= int($self->{max_children} * 0.2);
            $self->{min_spare} ||= int($self->{max_children} * 0.8);
        }
        $self->{min_spare} ||=1;
        $self->{max_spare} ||=2;
    }
    else {
        $self->{min_spare} ||= $self->{start_children};
        $self->{max_spare} ||= 2*$self->{min_spare};
        $self->{max_children} = $self->{start_children} 
                                    + $self->{max_spare};
    }
    DEBUG and warn "min_spare=$self->{min_spare} max_spare=$self->{max_spare} max_children=$self->{max_children}";
}

########################################################
sub _start
{
    my($self, $kernel)=@_[OBJECT, KERNEL];

    DEBUG and warn "Alias for ".ref($self)." is $self->{alias}";
    $kernel->alias_set($self->{alias});
    $Daemon::alias=$self->{alias};

    $kernel->sig(CHLD => 'sig_CHLD');
    $kernel->sig(INT  => 'sig_INT'); 

    $kernel->signal($kernel, 'daemon_start');

    ####
    if($self->is_prefork) {                 # pre-forking
        $self->default_min_max;
    }
    elsif($self->is_fork) {                 # forking
    } 
    else {
        $kernel->yield('waste_time');       # keep the daemon alive
        return;                             # and do nothing else
    }
                                            
    ####                                    # keep track of children
    $self->{children} = {};
    $self->{'failed forks'} = [];
    $self->{verbose}||=DEBUG;
    $self->{"max requests"}=$self->{requests}||1;
    $self->{'is a child'} = 0;              # change behavior in child
    $self->{scoreboard}=
            POE::Component::Daemon::Scoreboard->new($self->{max_children});
    $self->{'pending forks'} = 0;

    ####
    if($self->is_prefork) {
        DEBUG and warn "Pre-forking children";
        $self->{startup}=1;
        # fork the initial set of children
        $self->fork_off($self->{start_children});
    }
    ####
    if($self->{babysit}) {
        $self->{"proctable"}=eval {
            require Proc::ProcessTable;
            return new Proc::ProcessTable;
        };
        DEBUG and do {
            warn "Unable to load Proc::ProcessTable: $@" if $@;
        };
        $kernel->yield('babysit');
    }
    ####
    $kernel->yield('waste_time');       # keep the daemon alive
    return;
}

########################################################
# This event keeps this POE kernel alive
sub waste_time
{
    my($self, $kernel)=@_[OBJECT, KERNEL];
    return if $self->{'is a child'};

    DEBUG and warn "Still alive!";

    unless($self->{'been told we are parent'}) {
        DEBUG and warn "Telling everyone we are the parent";
        $self->{'been told we are parent'}=1;
        $kernel->signal($kernel, 'daemon_parent');
    }
    if($self->{'die'}) {
        DEBUG and warn "Orderly shutdown";
    } else {
        $kernel->delay('waste_time', 600);  # TODO : configable
    }
    return;
}

########################################################
# Babysit the child processes
sub babysit
{
    my($self, $kernel)=@_[OBJECT, KERNEL];

    return if $self->{'die'} or             # don't scan if we are dieing
              $self->{'is a child'};        # or if we are a child

    my @children=keys %{$self->{children}};
    ($self->{verbose} or DEBUG) and 
            warn "Babysiting ", scalar(@children),
                            " children ", join(", ", sort @children);
    my %table;

    if($self->{proctable}) {
        my $table=$self->{proctable}->table;
        %table=map {($_->pid, $_)} @$table  
    }

    my(%missing, $state, $time, %rogues, %ok);
    foreach my $pid (@children) {
        if($table{$pid}) {
            $state=$table{$pid}->state;

            if($state eq 'zombie') {
                my $t=waitpid($pid, POSIX::WNOHANG());
                if($t==$pid) {
                    # process was reaped, now fake a SIGCHLD
                    DEBUG and warn "Faking a CHLD for $pid";
                    $kernel->yield('sig_CHLD', 'CHLD', $pid, $?, 1);
                    $ok{$pid}=1;
                } else {
                    $self->{verbose} and warn "$pid is a $state and couldn't be reaped.";
                    $missing{$pid}=1;
                }
            }    
            elsif($state eq 'run') {
                $time=eval{$table{$pid}->utime + $table{$pid}->stime};
                warn $@ if $@;
                # utime and stime are Linux-only :(

                if($time and $time > 600000) { # arbitrary limit of 10 minutes
                    $rogues{$pid}=$table{$pid};
                    # DEBUG and 
                        warn "$pid has gone rogue, time=$time ms";
                } else {
                    warn "$pid time=$time ms";
                    $ok{$pid}=1;
                }

            } elsif($state eq 'sleep' or $state eq 'defunct') {
                $ok{$pid}=1;
                # do nothing
            } else {
                $self->{verbose} and warn "$pid has unknown state '$state'";
                $ok{$pid}=1;
            }
        } 
        elsif($self->{proctable}) {
            $self->{verbose} and warn "$pid isn't in proctable!";
            $missing{$pid}=1;
        } 
        else {                        # try another means.... :/
            if(-d "/proc" and not -d "/proc/$pid") {
                DEBUG and warn "Unable to stat /proc/$pid!  Is the child missing";
                $missing{$pid}=1;
            } 
            elsif(not $missing{$pid}) {
                $ok{$pid}=1;
            }
        }
    }

    # if a process is MIA, we fake a death, and spawn a new child (if needs be)
    foreach my $pid (keys %missing) {
        $kernel->yield('sig_CHLD', 'CHLD', $pid, 0, 1);
        $self->{verbose} and warn "Faking a CHLD for $pid MIA";
    }

    # we could do the same thing for rogue processes, but instead we
    # give them time to calm down

    if($self->{rogues}) {           # processes that are %ok are now removed
                                    # from the list of rogues
        delete @{$self->{rogues}}{keys %ok} if %ok;
    }

    if(%rogues) {
        $kernel->yield('rogues') if not $self->{rogues};

        $self->{rogues}||={};
        foreach my $pid (keys %rogues) {
            if($self->{rogues}{$pid}) { 
                $self->{rogues}{$pid}{proc}=$rogues{$pid};
            } else {
                $self->{rogues}{$pid}={proc=>$rogues{$pid}, tries=>0};
            }
        }
    }

    $kernel->delay('babysit', $self->{babysit});
    return;
}

########################################################
# Deal with rogue child processes
sub rogues
{
    my($self, $kernel)=@_[OBJECT, KERNEL];

    return if $self->{'die'} or             # don't scan if we are dieing
              $self->{'is a child'};        # or if we are a child

    return unless $self->{rogues};          # make sure we have some real work
    eval {
        if(ref($self->{rogues}) ne 'HASH' or not keys %{$self->{rogues}}) {
            delete $self->{rogues};
            return;
        }

        my $signal;
        while(my($pid, $rogue)=each %{$self->{rogues}}) {
            $signal=0;
            if($rogue->{tries} < 1) {       $signal=SIGINT;  }
            elsif($rogue->{tries} < 2) {    $signal=SIGTERM; }   
            elsif($rogue->{tries} < 3) {    $signal=SIGKILL;  }
    
            if($signal) {
                DEBUG and warn "Sending signal $signal to rogue $pid";
                unless($rogue->{proc}->kill($signal)) {
                    warn "Error sending signal $signal to $pid: $!";
                    delete $self->{rogues}{$pid};
                }
            } 
            else {
                # if SIGKILL didn't work, it's beyond hope!
                $kernel->yield('sig_CHLD', 'CHLD', $pid, 0, 1);
                delete $self->{rogues}{$pid};
                $self->{verbose} and warn "Faking a CHLD for rogue $pid";
            }
            $rogue->{tries}++;
        }
        $kernel->delay('rogues', 2*$self->{babysit});
    };
    warn "$$: $@" if $@;
    return;
}

########################################################
# Accept POE's standard _stop event, and stop all the children, too.
# The 'children' hash is maintained in the 'fork' and 'sig_CHLD'
# handlers.  It's empty for children.
sub _stop
{
    my($self, $kernel)=@_[OBJECT, KERNEL];
    $Daemon::alias='';

    DEBUG and warn "Server is stoping";
    # DEBUG_USR2 and check_kernel($kernel, $self->{'is a child'}, 1);
}

########################################################
# Someone wants us to exit... oblige
sub shutdown
{
    my($self, $kernel)=@_[OBJECT, KERNEL];

    DEBUG and warn "shutdown";
    if($self->{rogues}) {
        $kernel->delay('rogues');   # we no longer care about rogues
    }

    if($self->{children}) {         # tell children to go away
        foreach my $pid (keys %{$self->{children}}) {
            kill SIGINT, $pid
                    or warn "Killing $pid: $!";
        }
    }

    if(defined $self->{'my slot'}) {        # notice in the scoreboard
        $self->{scoreboard}->write($self->{'my slot'}, 'e');
        delete $self->{'my slot'};
    }

    $kernel->alias_remove($self->{alias});
    $kernel->delay('waste_time');   # get it OVER with
    $self->{'die'}=1;               # prevent race conditions
    $kernel->signal($kernel, 'daemon_shutdown');
    return;
}

########################################################
# The server has been requested to fork, so fork already.
sub fork 
{
    my ($kernel, $self, $req) = @_[KERNEL, OBJECT, ARG0];

    # children should not honor this event
    # Note that the forked POE kernel might have these events in it already
    # This is unavoidable :-(
    if($self->{'is a child'}) {
        return;
    }
    return if not $self->{children} or $self->{'die'};

    ####
    DEBUG and warn "pending forks=$self->{'pending forks'}";
    if($self->{"pending forks"}) {
        $self->{"pending forks"}--;
    }

    ####
    if($self->{max_children} <= keys %{$self->{children}}) {
        warn "Maximum number of children reached!";
        warn Dumper $self;
        return;
    }

    my $slot=$self->{scoreboard}->add('FORK');  # grap a slot in scoreboard

    DEBUG and warn "Forking a child";
    my $pid = fork();                   # try to fork
    unless (defined($pid)) {            # did the fork fail?
        $self->{scoreboard}->drop($slot);   # give slot back
        $self->fork_failed($!, "$!", $req);
        return;
    }

    if ($pid) {                         # successful fork; parent keeps track
        $self->{children}->{$pid} = $slot;
        DEBUG and
            warn "Parent server forked a new child.  children: (",
                    join(' ', sort keys %{$self->{children}}), ")";

        if(not $self->{"pending forks"} and $self->{startup}) {
            $self->{startup}=0;
            $kernel->yield('check_scoreboard');
        }
    }
    else {                              # child becomes a child process
        $self->{scoreboard}->write($slot, 'fork');
        $self->become_child($slot, $req);
    }
    return;
}

########################################################
# We failed to fork!
sub fork_failed
{
    my($self, $errnum, $errstr, $req)=@_;
    if (($errnum == EAGAIN) || ($errnum == ECHILD)) {
                                    # try again later, if a temporary error
        DEBUG and warn "Recoverable forking problem";
        push @{$self->{'failed forks'}}, $req;
        $poe_kernel->delay('retry', 1);
    }
    else {                          # fail permanently, if fatal
        warn "Can't fork: $errstr";
        $poe_kernel->yield('_stop');
    }
    return;
}

########################################################
# Turn ourselves into a child process
sub become_child
{
    my($self, $slot, $req)=@_;

    $self->{verbose} and warn "Created ", scalar localtime;
    # Clean out stuff that the parent needs but not the children

    $self->{'is a child'}   = 1;        # don't allow fork
    $self->{'my slot'}      = $slot;
    delete $self->{'pending forks'};
    delete $self->{'failed forks'};

    $poe_kernel->sig('CHLD');
    $poe_kernel->sig('INT'); 
    # remove the wait for babysit
    $poe_kernel->delay('babysit') if $self->{'babysit'};
    # remove the wait for checking the scorebard
    $poe_kernel->delay('check_scoreboard') if $self->is_prefork;
    # remove these fields
    delete @{$self}{qw(rogues proctable children)};

    # Tell everyone we are now a child
    DEBUG and warn "Telling everyone we are a child";
    $poe_kernel->signal($poe_kernel, 'daemon_child', $req);

    if($self->is_prefork) {
        $self->{requestN}=0;
    }
    elsif($self->is_fork) {
        $self->{scoreboard}->write($slot, 'req');
    }

    DEBUG and warn "Child server has been forked";
    return;
}

########################################################
# Retry failed forks.  This is invoked (after a brief delay) if the
# 'fork' state encountered a temporary error.
sub retry 
{
    my ($kernel, $self) = @_[KERNEL, OBJECT];
    if($self->{'is a child'} or not $self->{children}) {
        warn "We are a child, why are we forking?";
        return;
    }

    # Multiplex the delayed 'retry' event into enough 'fork' events to
    # make up for the temporary fork errors.
    $self->fork_off($self->{'failed forks'});
                                        # reset the failed forks counter
    $self->{'failed forks'} = [];
    return;
}


########################################################
# SIGCHLD causes this session to fork off a replacement for the lost child.
sub sig_CHLD
{
    my ($kernel, $self, $signal, $pid, $status, $fake) =
                @_[KERNEL, OBJECT, ARG0, ARG1, ARG2, ARG3];

    DEBUG and warn "SIGCHLD";
    return if $self->{"is a child"};

    ##########
    if($self->{foreign_children} and $self->{foreign_children}{$pid}) {
        DEBUG and warn "Foreign child $pid exited.";
        delete $self->{foreign_children};
        return;
    }

    ##########
    if($self->{children}) {
                                # if it was one of ours; fork another
        my $slot=delete $self->{children}->{$pid};
        if (defined $slot) {
            DEBUG and warn "Parent caught SIGCHLD for $pid.  children: (",
                                join(' ', sort keys %{$self->{children}}), 
                                ")";
            $self->{verbose} and warn "$$: Child $pid ",
                        ($fake?'is gone':'exited normaly');
            $self->{scoreboard}->drop($slot);
            if($self->is_prefork) {
#                $self->fork_off(1);
#                $kernel->delay('check_scoreboard'); # get rid of this delay
#                $kernel->yield('check_scoreboard'); # and do it right now
            }
        } 
        elsif($fake) {
            warn "Needless fake CHLD for $pid.";
        } 
        else {
            warn "CHLD for $pid child of someone else.";
            warn Dumper $self;
        }
    }

                                        # don't handle terminal signals
    return;
}

########################################################
# Terminal signals aren't handled, so the session will stop on SIGINT.
# The shutdown event handler takes care of cleanup.
sub sig_INT
{
    my ($kernel, $self, $signal, $pid, $status) =  
                @_[KERNEL, HEAP, ARG0, ARG1, ARG2];

    DEBUG and warn "SIGINT";
    $kernel->yield('shutdown');
    $kernel->sig_handled();
    return;
}

########################################################
# Start the process of creating a number of children
sub fork_off
{
    my($self, $n)=@_;
    if(ref $n) {
        foreach my $req (@$n) {
            $self->{"pending forks"}++;
            $poe_kernel->yield('fork', $req);
        }
        return;
    }
    
    for(my $q=0; $q < $n; $q++) {
        $self->{"pending forks"}++;
        $poe_kernel->yield('fork');
    }
    return;
}

########################################################
# Make sure we have min_spare waiting processes
# But no more than max_spare
sub check_scoreboard
{
    my($self)=@_;
    if($self->{'is a child'}) {
        warn "I am a child!  I refuse to check the scoreboard!";
        return;
    }
    # DEBUG_SC and warn "Checking scoreboard";

    my $slots=$self->{scoreboard}->read_all;
    my @waiting;                # PIDs of waiting children

    while(my($pid, $slot)=each %{$self->{children}}) {
        if($slots->[$slot] eq 'w') {      # waiting for req
            push @waiting, $pid;
        }
    }

    my $waiting=@waiting;
    # DEBUG_SC and warn "waiting=$waiting";
    if($waiting < $self->{min_spare}) {
        my $n=$self->{min_spare} - $waiting;
        DEBUG_SC and warn "Spawning $n spares";
        $self->fork_off($n);
    }
    if($waiting > $self->{max_spare}) {
        my $n=$waiting - $self->{max_spare};
        DEBUG_SC and warn "Killing $n spares";
        foreach my $pid (@waiting[0..$n]) {
            kill SIGINT, $pid or warn "killing $pid: $!";
        }
    }
    $poe_kernel->delay('check_scoreboard', 1);
}

########################################################
sub update_status
{
    my($self, $status, $parm)=@_[OBJECT, ARG0, ARG1];
    DEBUG and warn "Update status status=$status";

    if($self->is_prefork) {
        return $self->update_status_prefork($status, $parm);
    }

    if($self->is_fork) {
        if($self->{'is a child'}) {
            return $self->update_status_fork_child($status, $parm);
        } 
        else {
            return $self->update_status_fork_parent($status, $parm);
        }
    }
    warn "Non-forking server doesn't need to update status";
    return;
}

########################################################
sub update_status_prefork
{
    my($self, $status, $parm)=@_;
    unless($self->{'is a child'}) {
        warn "Only child processes should update their status";
        return;
    }

    my $slot=$self->{'my slot'};
    if($status eq 'wait' or $status eq 'done') {
        $self->{scoreboard}->write($slot, 'wait');
        if($self->{requestN} >= $self->{'max requests'}) {
            DEBUG and warn "Handled $self->{requestN} requests, shutting down";
            $poe_kernel->yield('shutdown');
        }
        return;
    }

    if($status eq 'long') {
        DEBUG and warn "Long request $self->{requestN}";
        my $s=$self->{scoreboard}->read($slot);
        if($s eq 'r') {
            $self->{scoreboard}->write($slot, 'long');
            return;
        }
        # allow to fall through if we didn't get a 'req' previous
        # for now this isn't important, but it might be later
    }

    if($status eq 'req' or $status eq 'long') {
        $self->{requestN}++;
        DEBUG and warn "Handling request $self->{requestN}";
        $self->{scoreboard}->write($slot, $status);
        return;
    }
    warn "Don't know what do for '$status'! Maybe you meant 'long', 'wait' or 'done' or 'req'?";
}


########################################################
sub update_status_fork_child
{
    my($self, $status, $parm)=@_;

    my $slot=$self->{'my slot'};
    die "NO SLOT" unless defined $slot;

    if($status eq 'wait' or $status eq 'done') {
        $self->{scoreboard}->write($slot, 'wait');
        $poe_kernel->yield('shutdown');
        return;
    }

    if($status eq 'long') {
        $self->{scoreboard}->write($slot, $status);
        return;
    }

    warn "Don't know what to do for '$status'! Maybe you meant 'long', 'wait' or 'done'?";
}

########################################################
sub update_status_fork_parent
{
    my($self, $status, $parm)=@_;

    if($status eq 'req') {
        $self->fork_off([$parm]);
        return;
    }

    warn "Don't know what to do for '$status'! Maybe you meant 'req'?";
}



########################################################
sub status
{
    my($self)=@_;
    my @ret=ref($self);

    my $q='the parent';
    $q='a child' if $self->{'is a child'};
    if($self->is_prefork) {
        push @ret, "Pre-forking server, we are $q";
    }
    elsif($self->is_fork) {
        push @ret, "Forking server, we are $q";
    }
    else {
        push @ret, "Daemon server";
    }

    if($self->{children}) {
        my $kids=$self->{children};
        push @ret, (0+keys %$kids), " children: ", sort join ' ', keys %$kids;

        my $doing=$self->{scoreboard}->read_all;
        foreach my $pid (sort keys %$kids) {
            push @ret, "        $pid: $doing->[$kids->{$pid}]";
        }
    }
    return join("\n    ", @ret)."\n".$self->{scoreboard}->status;
}

##########################################################################
package Daemon;

use strict;
use POE;

use vars qw($alias);

sub update_status
{
    my($package, $status, $parm)=@_;
    $poe_kernel->call($alias => 'update_status', $status, $parm);
}

sub forked
{
    my($package, $pid)=@_;
    $poe_kernel->call($alias => 'forked', $pid);
}

sub shutdown
{
    $poe_kernel->call($alias => 'shutdown');
}

sub status
{
    return $poe_kernel->call($alias => 'status');
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

POE::Component::Daemon - Handles all the housework for a daemon.

=head1 SYNOPSIS

  use POE::Component::Daemon;

  POE::Component::Daemon->spawn(detach=>1, max_children=>3);


    # Create a session that uses SocketFactory
    POE::Session->create(
    inline_states=>{

        _start=>sub {
            # catch this message from Daemon session
            $kernel->sig('daemon_child'=>'request');

            # create a POE::Wheel::SocketFactory or whatever
            # .....
        },

        # socketfactory got a connection handle it here
        accept=>sub {       
            # tell Daemon session about this
            Daemon->update_status('req', $info);
        },

        ###############
        # we are now the child process (via the sig() in _start
        request=>sub {
            my($heap, $info)=@_[HEAP, ARG1]; 
            # $info was passed here from accept accept

            # create POE::Wheel::ReadWrite
            ....
            # tell Daemon session that this request will take a long time
            Daemon->update_status('long');
        },

        ###############
        # The request is finished
        finished=>sub {
            return unless $heap->{done};
            # tell Deamon session that this request is done
            $poe_kernel->post(Daemon=>'update_status', 'done');
        },
    });      

=head1 ABSTRACT

POE::Component::Daemon is a module that will deal with all the niggling
details of creating and maintaining a daemon process.  It can detach from the
console, handle pre-forking pools or post-forking (ie, fork on each request).
It will also redirect STDERR to a log file if asked.


=head1 DESCRIPTION

POE::Component::Daemon is a module that will deal with all the niggling
details of creating and maintaining a daemon process.  It can detach from
the console, handle pre-forking pools or post-forking (ie, fork on each
request). It will also redirect STDERR to a log file if asked.  

POE::Component::Daemon also babysits child processes, handling their SIGCHLD.  POE::Component::Daemon can
also make requestions don't take to long.  If they do, it will try to kill
them and so on.  See L<BABYSITING> below.

POE::Component::Daemon does not handle listen on sockets and so on.

Like all of POE, POE::Component::Daemon works cooperatively.  It is up your code to
tell POE::Component::Daemon when it is time to fork, block incoming requests when
approriate and so on.  

=head2 POST-FORKING

Post-forking is the model that most examples and tutorials use.  The daemon
listens on a socket (or other mechanism) for new requests.  When a new
request comes in, a child process is forked off to handle that request and
the parent process continues to listen for new requests.

If you are using a post-forking model, your code must inform POE::Component::Daemon about a
new request.  POE::Component::Daemon will then handle all the details of forking, and then
broadcast a daemon_child signal, which is your cue that you can now handle
the request.

This means the following steps are done.

    create SocketFactory
    create POE::Component::Daemon
    receive SocketFactory's SuccessEvent
    tell POE::Component::Daemon we are in a request
    POE::Component::Daemon forks
    POE::Component::Daemon sends daemon_child signal
    receive daemon_child signal, create ReadWrite wheel
    talk with remote process
    when done, close ReadWrite wheel
    tell POE::Component::Daemon we are no longer in a request
    POE::Component::Daemon will then shutdown this process.
    
=head2 PRE-FORKING

The pre-forking model creates a pool of child processes before accepting
requests.  This is done so that each request doesn't incure the overhead of
forking before it can be processed.  It also allows a child process to
handle more then one request.  This is the model used by Apache.

When pre-forking, you should create your listen socket and immediately pause
it.  Then spawn a POE::Component::Daemon, which will fork off the desired number of
sub-processes.

Process pools are maintained with the help of a scoreboard.  You may 
view the scoreboard with the status() method

=head1 BABYSITING

=head1 CALL SYNTAX

=head1 METHODS

=head2 spawn

=over 4

=item alias
=item detach
=item logfile
=item max_children
=item verbose

=item max_spare
=item min_spare
=item requests
=item start_children

=back

=head2 status

=head1 SCOREBOARD

=over 4

=item ' '

Slot is empty.

=item e

Process is exiting.

=item F

Process is forking, but we are still in the parent

=item f

Process is forking, we are in the child.

=item .

Process is waiting for first request.

=item r

Process is handling a request.

=item l

Process is handling a long request.

=item w

Process is waiting for next request.

=back


=head1 SIGNALS

=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Philip Gwyn, E<lt>fil@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by Philip Gwyn

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut

$Log: Daemon.pm,v $
Revision 1.2  2004/08/19 01:18:22  fil
added doco

